@@ -319,44 +319,6 @@ tracker_file_system_get_remaining_space_percentage (const gchar *path)
 	}
 }
 
-gboolean
-tracker_file_system_has_enough_space (const gchar *path,
-                                      gulong       required_bytes,
-                                      gboolean     creating_db)
-{
-	gchar *str1;
-	gchar *str2;
-	gboolean enough;
-	guint64 remaining;
-
-	g_return_val_if_fail (path != NULL, FALSE);
-
-	remaining = tracker_file_system_get_remaining_space (path);
-	enough = (remaining >= required_bytes);
-
-	if (creating_db) {
-		str1 = g_format_size (required_bytes);
-		str2 = g_format_size (remaining);
-
-		if (!enough) {
-			g_critical ("Not enough disk space to create databases, "
-			            "%s remaining, %s required as a minimum",
-			            str2,
-			            str1);
-		} else {
-			g_debug ("Checking for adequate disk space to create databases, "
-			         "%s remaining, %s required as a minimum",
-			         str2,
-			         str1);
-		}
-
-		g_free (str2);
-		g_free (str1);
-	}
-
-	return enough;
-}
-
 gboolean
 tracker_path_is_in_path (const gchar *path,
                          const gchar *in_path)
@@ -649,87 +611,6 @@ tracker_path_evaluate_name (const gchar *path)
 	return final_path;
 }
 
-static gboolean
-path_has_write_access (const gchar *path,
-                       gboolean    *exists)
-{
-	GFile     *file;
-	GFileInfo *info;
-	GError    *error = NULL;
-	gboolean   writable;
-
-	g_return_val_if_fail (path != NULL, FALSE);
-	g_return_val_if_fail (path[0] != '\0', FALSE);
-
-	file = g_file_new_for_path (path);
-	info = g_file_query_info (file,
-	                          G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE,
-	                          0,
-	                          NULL,
-	                          &error);
-
-	if (G_UNLIKELY (error)) {
-		if (error->code == G_IO_ERROR_NOT_FOUND) {
-			if (exists) {
-				*exists = FALSE;
-			}
-		} else {
-			gchar *uri;
-
-			uri = g_file_get_uri (file);
-			g_warning ("Could not check if we have write access for "
-			           "'%s': %s",
-			           uri,
-			           error->message);
-			g_free (uri);
-		}
-
-		g_error_free (error);
-
-		writable = FALSE;
-	} else {
-		if (exists) {
-			*exists = TRUE;
-		}
-
-		writable = g_file_info_get_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE);
-
-		g_object_unref (info);
-	}
-
-	g_object_unref (file);
-
-	return writable;
-}
-
-gboolean
-tracker_path_has_write_access_or_was_created (const gchar *path)
-{
-	gboolean writable;
-	gboolean exists = FALSE;
-
-	writable = path_has_write_access (path, &exists);
-	if (exists) {
-		if (writable) {
-			g_message ("  Path is OK");
-			return TRUE;
-		}
-
-		g_message ("  Path can not be written to");
-	} else {
-		g_message ("  Path does not exist, attempting to create...");
-
-		if (g_mkdir_with_parents (path, 0700) == 0) {
-			g_message ("  Path was created");
-			return TRUE;
-		}
-
-		g_message ("  Path could not be created");
-	}
-
-	return FALSE;
-}
-
 gboolean
 tracker_file_is_hidden (GFile *file)
 {
