@@ -251,7 +251,8 @@ tag_to_string (TIFF    *image,
 }
 
 G_MODULE_EXPORT gboolean
-tracker_extract_get_metadata (TrackerExtractInfo *info)
+tracker_extract_get_metadata (TrackerExtractInfo  *info,
+                              GError             **error)
 {
 	TrackerResource *metadata;
 	TIFF *image;
@@ -284,15 +285,20 @@ tracker_extract_get_metadata (TrackerExtractInfo *info)
 	fd = tracker_file_open_fd (filename);
 
 	if (fd == -1) {
-		g_warning ("Could not open tiff file '%s': %s\n",
-		           filename,
-		           g_strerror (errno));
+		g_set_error (error,
+		             G_IO_ERROR,
+		             g_io_error_from_errno (errno),
+		             "Could not open tiff file: %s",
+		             g_strerror (errno));
 		g_free (filename);
 		return FALSE;
-	}	
+	}
 
 	if ((image = TIFFFdOpen (fd, filename, "r")) == NULL){
-		g_warning ("Could not open image:'%s'\n", filename);
+		g_set_error (error,
+		             G_IO_ERROR,
+		             G_IO_ERROR_INVALID_ARGUMENT,
+		             "Could not parse tiff file");
 		g_free (filename);
 		close (fd);
 		return FALSE;
