@@ -61,7 +61,8 @@ class MinerCrawlTest(fixtures.TrackerMinerTest):
             for tf in monitored_files:
                 url = self.uri(tf)
                 self.tracker.ensure_resource(fixtures.DOCUMENTS_GRAPH,
-                                             f"a nfo:Document ; nie:isStoredAs <{url}>")
+                                             f"a nfo:Document ; nie:isStoredAs <{url}>",
+                                             timeout=cfg.AWAIT_TIMEOUT)
         except Exception:
             cfg.remove_monitored_test_dir(self.workdir)
             raise
@@ -331,10 +332,16 @@ class MinerCrawlTest(fixtures.TrackerMinerTest):
         with self.tracker.await_delete(fixtures.DOCUMENTS_GRAPH, file_inside_victim_id, timeout=cfg.AWAIT_TIMEOUT):
             shutil.rmtree(victim)
 
+        # We have 2 text files inside the directory, but only look for one, assert that
+        # the other file is gone as well.
+        file_inside_victim_url2 = self.uri(os.path.join(victim, "dir2/file3.txt"))
+        counter = 0
+        while counter < 10 and self.tracker.ask("ASK { <%s> a nfo:FileDataObject }" % (file_inside_victim_url2)):
+            counter += 1
+            time.sleep(1)
+
         result = self.__get_text_documents()
-        self.assertEqual(len(result), 1)
-        unpacked_result = [r[0] for r in result]
-        self.assertIn(self.uri("test-monitored/file1.txt"), unpacked_result)
+        self.assertEqual(result, [[self.uri("test-monitored/file1.txt")]])
 
         # Restore the dirs
         os.makedirs(self.path("test-monitored/dir1"))
@@ -367,7 +374,7 @@ class MinerCrawlTest(fixtures.TrackerMinerTest):
             # indirectly by the new file)
             with open(document, 'w') as f:
                 f.write(DEFAULT_TEXT)
-            self.miner_fs.index_file(directory_uri)
+            self.miner_fs.index_location(directory_uri, [], [])
 
         new_urn = self.__get_file_urn(directory)
         # Ensure that children remain consistent, old and new ones
